import peek from 'it-peekable';
function isAsyncIterable(thing) {
    return thing[Symbol.asyncIterator] != null;
}
function map(source, func) {
    if (isAsyncIterable(source)) {
        return (async function* () {
            for await (const val of source) {
                yield func(val);
            }
        })();
    }
    // if mapping function returns a promise we have to return an async generator
    const peekable = peek(source);
    const { value, done } = peekable.next();
    if (done === true) {
        return (function* () { }());
    }
    const res = func(value);
    // @ts-expect-error .then is not present on O
    if (typeof res.then === 'function') {
        return (async function* () {
            yield await res;
            for await (const val of peekable) {
                yield func(val);
            }
        })();
    }
    const fn = func;
    return (function* () {
        for (const val of source) {
            yield fn(val);
        }
    })();
}
export default map;
//# sourceMappingURL=index.js.map